/************************************************************************
* File:        COLOR.C
* Description: Color support module. Contains 256 color palette accessing,
*              Best color matching, and our translucency routines.
* Author:      Jesse Towner.
************************************************************************/


/* I N C L U D E S *****************************************************/


#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include <stdio.h>
#include <dos.h>

#include "graphics.h"
#include "color.h"

#ifdef __cplusplus
extern "C" {
#endif


/* F U N C T I O N S ***************************************************/


void GFX_WaitForRetrace(void)
/************************************************************************
* Function:    GFX_WaitForRetrace
*
* Description: Waits for a vertical retrace on the VGA card to finish, at
*              which point the function returns.
************************************************************************/
{
 while (inp(0x03DA) & 0x08) {
	}
 while (!(inp(0x03DA) & 0x08)) {
	}
}


void GFX_SetPalette(gfxPalette_t pal, int startColor, int endColor, int vsync)
/************************************************************************
* Function:    GFX_SetPalette
*              pal        - 256 color palette to set
*              startColor - starting index color
*              endColor   - ending index color
*              vsync      - wether or not wait for vertical retrace
*
* Description: Sets the palette on VGA cards. If vsync != 0 then it will
*              wait for a vertical retrace to finish before setting the
*              palette.
************************************************************************/
{
 int index;

 /* Do we have to wait for a vertical retrace */
 if (vsync)
	GFX_WaitForRetrace();

 /* Set the palette data */
 for (index = startColor; index <= endColor; index++) {
	outp(0x3C8, index);
	outp(0x3C9, pal[index].r);
	outp(0x3C9, pal[index].g);
	outp(0x3C9, pal[index].b);
	}
}


void GFX_GetPalette(gfxPalette_t pal, int startColor, int endColor)
/************************************************************************
* Function:    GFX_GetPalette
*              pal        - 256 color palette to set
*              startColor - starting index color
*              endColor   - ending index color
*
* Description: Gets the palette on VGA cards.
************************************************************************/
{
 int index;

 /* Set the palette data */
 for (index = startColor; index <= endColor; index++) {
	outp(0x3C7, index);
	pal[index].r = inp(0x3C9);
	pal[index].g = inp(0x3C9);
	pal[index].b = inp(0x3C9);
	}
}


void GFX_SetColor(gfxRgb_t * rgb, int color)
/************************************************************************
* Function:    GFX_SetColor
*              rgb   - points to place to where RGB structure is.
*              color - color to set
*
* Description: Sets a color's RGB values on VGA cards.
************************************************************************/
{
 outp(0x3C8, color);
 outp(0x3C9, rgb->r);
 outp(0x3C9, rgb->g);
 outp(0x3C9, rgb->b);
}


void GFX_GetColor(gfxRgb_t * rgb, int color)
/************************************************************************
* Function:    GFX_GetColor
*              rgb   - points to place to store RGB values
*              color - color to get RGB values from
*
* Description: Gets a color's RGB values on VGA cards.
************************************************************************/
{
 outp(0x3C7, color);
 rgb->r = inp(0x3C9);
 rgb->g = inp(0x3C9);
 rgb->b = inp(0x3C9);
}


int GFX_FindBestColor(gfxPalette_t pal, int r, int g, int b)
/***********************************************************************
* Function:    GFX_FindBestColor
* Parameters:  pal   - the 256 color palette to search in
*              r,g,b - the r, g, b values. They must range from 0 to 63.
* Returns:     The value of the closest matching color.
*
* Description: Finds the closest matching color in the given palette for
*              a set of RGB values.
***********************************************************************/
{
 static unsigned long colorDifLut[3*128];
 static int firstTime = 1;
 int i, i2, colorDif, lowest = INT_MAX, bestMatch = 0;

 /* If this is the first time that this function has been called, then
    generate a little lookup table (aka LUT) of weighted squares. Color
    matching is done by taking the weighted square of the difference
    between a certain colors RGB values and the RGB values of the color
    we are searching for. */
 if (firstTime == 1) {
	firstTime = 0;
	for (i=0; i<64; i++) {
		int i2 = i * i;
		colorDifLut[0  +i] = colorDifLut[0  +128-i] = i2 * (59 * 59);
		colorDifLut[128+i] = colorDifLut[128+128-i] = i2 * (30 * 30);
		colorDifLut[256+i] = colorDifLut[256+128-i] = i2 * (11 * 11);
		}
	}

 /* Now, search through the palette for the best color. */
 for (i=0; i<256; i++) {
	colorDif = colorDifLut[(pal[i].g - g) & 0x7F];
	if (colorDif < lowest) {
		colorDif += (colorDifLut+128)[(pal[i].r - r) & 0x7F];
		if (colorDif < lowest) {
			colorDif += (colorDifLut+256)[(pal[i].b - b) & 0x7F];
			if (colorDif < lowest) {
				bestMatch = i;
				if (colorDif == 0)
					return bestMatch;
				else
					lowest = colorDif;
				}
			}
		}
	}
 return bestMatch;
}


#ifdef __cplusplus
};
#endif

/* end of file */