/************************************************************************
* File:        GRAPHICS.C
* Description: The source to our graphics module.
* Author:      Jesse Towner.
************************************************************************/


/* I N C L U D E S *****************************************************/


#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include <stdio.h>
#include <dos.h>

#include "graphics.h"
#include "color.h"
#include "trans.h"

#ifdef __cplusplus
extern "C" {
#endif


/* D A T A *************************************************************/


/* Pointer to the VGA Mode 13h screen */
unsigned char * gfxScreenPtr = (unsigned char *)(0xA0000UL);


/* F U N C T I O N S ***************************************************/


void GFX_SetMode13h(void)
/************************************************************************
* Function:    GFX_SetMode13h
*
* Description: Sets the video mode to VGA Mode 13h (320x200x256).
************************************************************************/
{
 union REGS regs;
 regs.w.ax = 0x0013;
 int386(0x10, &regs, &regs);
}


void GFX_SetTextMode(void)
/************************************************************************
* Function:    GFX_SetTextMode
*
* Description: Sets the video mode to text mode 03h (80x25x16).
************************************************************************/
{
 union REGS regs;
 regs.w.ax = 0x0003;
 int386(0x10, &regs, &regs);
}


unsigned char * GFX_CreateBuffer(void)
/************************************************************************
* Function:    GFX_CreateBuffer
*
* Description: Creates an offscreen buffer in RAM and returns a pointer
*              to it. The size in pixel is that of the screen size.
************************************************************************/
{ return (unsigned char *)malloc(GFX_SCR_WIDTH * GFX_SCR_HEIGHT); }


void GFX_DestroyBuffer(unsigned char * buffer)
/************************************************************************
* Function:    GFX_DestroyBuffer
* Parameters:  buffer - pointer to the buffer to destroy
*
* Description: De-allocates an offscreen buffer that was previously
*              created by GFX_CreateBuffer().
************************************************************************/
{ free(buffer); }


gfxBitmap_t * GFX_CreateBitmap(int width, int height, int bpp)
/************************************************************************
* Function:    GFX_CreateBitmap
* Parameters:  width  - the width in pixel of bitmap
*              height - the height in pixels of bitmap
*              bpp    - the bits per pixel of bitmap
* Returns:     A pointer to the allocated bitmap structure.
*
* Description: Creates a bitmap based on the passed informatiob and
*              then it returns a pointer to it.
************************************************************************/
{
 gfxBitmap_t * bitmap;
 unsigned long bufSize;

 /* first, allocate the structure */
 if ((bitmap = (gfxBitmap_t *)malloc(sizeof(gfxBitmap_t))) == NULL)
	return NULL;

 /* setup the bitmap */
 bitmap->width = width;
 bitmap->height = height;
 bitmap->bpp = bpp;

 /* Now allocate the bitmap's surface */
 bufSize = (width * height * ((bpp == 15) ? (2) : (bpp >> 3)));

 if ((bitmap->surface = (void *)malloc(bufSize)) == NULL) {
	free(bitmap);
	return NULL;
	}

 /* Return pointer to the bitmap */
 return bitmap;
}


void GFX_DestroyBitmap(gfxBitmap_t * bitmap)
/************************************************************************
* Function:    GFX_CreateBitmap
* Parameters:  bitmap - the bitmap to de-allocate
*
* Description: Destroys a bitmap structure that was previosly allocated
*              by a call to GFX_CreateBitmap().
************************************************************************/
{
 free(bitmap->surface);
 free(bitmap);
}


void GFX_ClrScr(unsigned char * buffer, int color)
/************************************************************************
* Function:    GFX_ClrScr
* Parameters:  buffer - pointer to the screen or off-screen buffer
*              color  - the color to clear with
*
* Description: Clears the screen or an off-screen buffer to the passed
*              color value.
************************************************************************/
{ memset(buffer, color, GFX_SCR_WIDTH * GFX_SCR_HEIGHT); }


void GFX_BitBlt(unsigned char * dest, unsigned char * source)
/************************************************************************
* Function:    GFX_BitBlt
* Parameters:  dest   - pointer to the destination buffer to blit to
*              source - pointer to the source buffer to blit from
*
* Description: Copys the contents of one buffer (screen/off-screen) to
*              to another.
************************************************************************/
{ memcpy(dest, source, GFX_SCR_WIDTH * GFX_SCR_HEIGHT); }


void GFX_PutPixel(unsigned char * buffer, int x, int y, int color)
/************************************************************************
* Function:    GFX_PutPixel
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x, y   - coordinates to plot the pixel at
*              color  - color of the pixel to plot
*
* Description: Puts a pixel on the screen or off-screen buffer at <X,Y>
*              using the passed color.
************************************************************************/
{
 /* Preform clipping on the screen boudaries. */
 if ((x < 0) || (x >= GFX_SCR_WIDTH) || (y < 0) || (y >= GFX_SCR_HEIGHT))
	return;

 /* write the pixel */
 buffer[(y * GFX_SCR_WIDTH) + x] = color;
}


void GFX_PutPixelTrans(unsigned char * buffer, int x, int y, int color)
/************************************************************************
* Function:    GFX_PutPixel
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x, y   - coordinates to plot the pixel at
*              color  - color of the pixel to plot
*
* Description: Puts a translucent, alpha blended pixel on the screen or
*              off-screen buffer at <X,Y> using the passed color.
************************************************************************/
{
 unsigned int color2;

 /* Preform clipping on the screen boudaries. */
 if ((x < 0) || (x >= GFX_SCR_WIDTH) || (y < 0) || (y >= GFX_SCR_HEIGHT))
	return;

 /* First, read the destination pixel that we would normally over-write */
 color2 = buffer[(y * GFX_SCR_WIDTH) + x];

 /* Blend the two colors together and place the result in color2 */
 color2 = gfxActiveClut->data[color][color2];

 /* write out the blended pixel */
 buffer[(y * GFX_SCR_WIDTH) + x] = color2;
}


void GFX_PutPixelTransASM(unsigned char * buffer, int x, int y, int color)
/************************************************************************
* Function:    GFX_PutPixelASM
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x, y   - coordinates to plot the pixel at
*              color  - color of the pixel to plot
*
* Description: Puts a translucent, alpha blended pixel on the screen or
*              off-screen buffer at <X,Y> using the passed color. This
*              is the assembly version.
************************************************************************/
{
 /* Preform clipping on the screen boudaries. */
 if ((x < 0) || (x >= GFX_SCR_WIDTH) || (y < 0) || (y >= GFX_SCR_HEIGHT))
	return;

 _asm {
 /* First, setup EDI so that it points to the pixel address */
	mov  edi,y
	mov  eax,y
	shl  edi,6
	shl  eax,8
	add  edi,eax 		    /* EDI = ((y << 6) + (y << 8)); */
	add  edi,buffer         /* EDI += buffer; */
	add  edi,x              /* EDI += x; */

 /* Next, setup EBX so that it points to the active translucency CLUT */
	mov  ebx,color          /* Setup EBX so that it indexes into the color */
	shl  ebx,8
	add  ebx,gfxActiveClut  /* EBX = &gfxActiveClut->data[color][0]; */

 /* Finally, preform the actual blending of the pixel colors */
	xor  eax,eax            /* Zero out EAX */
	mov  al,[edi]           /* Read the pixel we would normally overwrite */
	mov  al,[ebx+eax]       /* AL = gfActiveClut->data[color][EAX]; */
	mov  [edi],al           /* Write out the blended color */
	};
}


void GFX_Rectangle(unsigned char * buffer, int x1, int y1, int x2, int y2,
	int color)
/************************************************************************
* Function:    GFX_Rectangle
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x1, y1 - coordinates of the upper left-hand corner
*              x2, y2 - coordinates of the lower right-hand corner
*              color  - color of the rectangle to draw
*
* Description: Draws a Rectangle on the screen or buffer from <x1,y1> to
*              <x2,y2> using the passed color.
************************************************************************/
{
 int width, height;
 unsigned char * drawPtr;

 /* Swap x1 with x2 if x1 is greater than x2 */
 if (x1 > x2) {
	x1 ^= x2;
	x2 ^= x1;
	x1 ^= x2;
	}

 /* Swap y1 with y2 if y1 is greater than y2 */
 if (y1 > y2) {
	y1 ^= y2;
	y2 ^= y1;
	y1 ^= y2;
	}

 /* Clip to the left clipping edge */
 if (x1 < 0) {
	if (x2 < 0)
		return;
	x1 = 0;
	}

 /* Clip to the right clipping edge */
 if (x2 >= GFX_SCR_WIDTH) {
	if (x1 >= GFX_SCR_WIDTH)
		return;
	x2 = GFX_SCR_WIDTH - 1;
	}

 /* Clip to the top clipping edge */
 if (y1 < 0) {
	if (y2 < 0)
		return;
	y1 = 0;
	}

 /* Clip to the bottom clipping edge */
 if (y2 >= GFX_SCR_HEIGHT) {
	if (y1 >= GFX_SCR_HEIGHT)
		return;
	y2 = GFX_SCR_HEIGHT-1;
	}

 /* Calculate the width and the height */
 width = x2 - x1 + 1;
 height = y2 - y1 + 1;

 /* Calculate the address of the first pixel to draw */
 drawPtr = buffer + (y1 * GFX_SCR_WIDTH) + x1;

 /* Draw the rectangle scan-line by scan-line */
 do	{
	memset(drawPtr, color, width);    /* Draw the scan-line */
	drawPtr += GFX_SCR_WIDTH;         /* Go to the next scan-line */
	} while (--height > 0);           /* Is there anymore scan-line to do? */
}


void GFX_RectangleTrans(unsigned char * buffer, int x1, int y1, int x2,
	int y2, int color)
/************************************************************************
* Function:    GFX_RectangleTrans
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x1, y1 - coordinates of the upper left-hand corner
*              x2, y2 - coordinates of the lower right-hand corner
*              color  - color of the rectangle to draw
*
* Description: Draws a translucent, alpha blended rectangle on the screen
*              or buffer from <x1,y1> to <x2,y2> using the passed color.
************************************************************************/
{
 unsigned int width, height, x;
 unsigned char * drawPtr;

 /* Swap x1 with x2 if x1 is greater than x2 */
 if (x1 > x2) {
	x1 ^= x2;
	x2 ^= x1;
	x1 ^= x2;
	}

 /* Swap y1 with y2 if y1 is greater than y2 */
 if (y1 > y2) {
	y1 ^= y2;
	y2 ^= y1;
	y1 ^= y2;
	}

 /* Clip to the left clipping edge */
 if (x1 < 0) {
	if (x2 < 0)
		return;
	x1 = 0;
	}

 /* Clip to the right clipping edge */
 if (x2 >= GFX_SCR_WIDTH) {
	if (x1 >= GFX_SCR_WIDTH)
		return;
	x2 = GFX_SCR_WIDTH-1;
	}

 /* Clip to the top clipping edge */
 if (y1 < 0) {
	if (y2 < 0)
		return;
	y1 = 0;
	}

 /* Clip to the bottom clipping edge */
 if (y2 >= GFX_SCR_HEIGHT) {
	if (y1 >= GFX_SCR_HEIGHT)
		return;
	y2 = GFX_SCR_HEIGHT-1;
	}

 /* Calculate the width and the height */
 width = x2 - x1 + 1;
 height = y2 - y1 + 1;

 /* Calculate the address of the first pixel to draw */
 drawPtr = buffer + (y1 * GFX_SCR_WIDTH) + x1;

 /* Draw the rectangle scan-line by scan-line */
 do	{
	/* For each pixel on the scanline... */
	for (x=x1; x<=x2; x++) {
		/* Obtain the destination pixel color and blend it with the
		   source color, then write that out */
		*drawPtr++ = gfxActiveClut->data[color][*drawPtr];
		}

	drawPtr += GFX_SCR_WIDTH;         /* Go to the next scan-line */
	drawPtr -= width;                 /* Make corrections for incrementing */
	} while (--height);               /* Is there anymore scan-line to do? */
}


void GFX_RectangleTransASM(unsigned char * buffer, int x1, int y1, int x2,
	int y2, int color)
/************************************************************************
* Function:    GFX_RectangleTransASM
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x1, y1 - coordinates of the upper left-hand corner
*              x2, y2 - coordinates of the lower right-hand corner
*              color  - color of the rectangle to draw
*
* Description: Draws a translucent, alpha blended rectangle on the screen
*              or buffer from <x1,y1> to <x2,y2> using the passed color.
*              This is the x86 assembly version.
************************************************************************/
{
 unsigned int width, height;
 unsigned char * drawPtr;

 /* Swap x1 with x2 if x1 is greater than x2 */
 if (x1 > x2) {
	x1 ^= x2;
	x2 ^= x1;
	x1 ^= x2;
	}

 /* Swap y1 with y2 if y1 is greater than y2 */
 if (y1 > y2) {
	y1 ^= y2;
	y2 ^= y1;
	y1 ^= y2;
	}

 /* Clip to the left clipping edge */
 if (x1 < 0) {
	if (x2 < 0)
		return;
	x1 = 0;
	}

 /* Clip to the right clipping edge */
 if (x2 >= GFX_SCR_WIDTH) {
	if (x1 >= GFX_SCR_WIDTH)
		return;
	x2 = GFX_SCR_WIDTH-1;
	}

 /* Clip to the top clipping edge */
 if (y1 < 0) {
	if (y2 < 0)
		return;
	y1 = 0;
	}

 /* Clip to the bottom clipping edge */
 if (y2 >= GFX_SCR_HEIGHT) {
	if (y1 >= GFX_SCR_HEIGHT)
		return;
	y2 = GFX_SCR_HEIGHT-1;
	}

 _asm {
 /* Calculate the height and width of our rectangle. We will store the
    height in EDX, and the width in EBX. */
	mov  ebx,x2
	mov  edx,y2
	sub  ebx,x1
	sub  edx,y1
	inc  ebx              /* EBX = x2 - x1 + 1 */
	inc  edx              /* EDX = y2 - y1 + 1 */

 /* Setup EDI so that it points to the first pixel to draw in the rect. */
	mov  eax,y1
	mov  edi,y1
	shl  eax,8
	shl  edi,6
	add  edi,eax
	add  edi,buffer
	add  edi,x1               /* EDI = ((y1<<6) + (y1<<8)) + buffer + x; */

 /* Now, setup ESI so that it indexes into the active CLUT table */
	mov  esi,color
	shl  esi,8
	add  esi,gfxActiveClut    /* ESI = &gfxActiveClut->data[color][0]; */
	xor  eax,eax              /* Zero out EAX */
	cld

 /* Now, lets draw the rectangle, line by line... */
 _Rect_Y_Loop:
	mov  ecx,ebx	           /* ECX = width */

 _Rect_X_Loop:
	mov  al,[edi]             /* Get destination pixel color */
	mov  al,[esi+eax]         /* AL = gfxActiveClut->data[color][EAX]; */
	mov  [edi],al             /* Write out the blended pixel */
	inc  edi                  /* Increment to the next pixel */
	dec  ecx                  /* Decrement the width count */
	jnz  _Rect_X_Loop         /* Is there another pixel to process? */

	add  edi,320              /* EDI += GFX_SCR_WIDTH; */
	sub  edi,ebx              /* EDI -= width; */
	dec  edx                  /* Decrement the height */
	jnz  _Rect_Y_Loop         /* Is there another scanline to process? */
	};
}


void GFX_PutImage(unsigned char * buffer, unsigned char * image, int x,
	int y, int width, int height)
/************************************************************************
* Function:    GFX_PutImage
* Parameters:  buffer - pointer to the buffer to draw the image onto
*              image  - pointer to the image data
*              x, y   - coordinates of the upper left-hand corner
*              width  - width, in pixels, of the image
*              height - height, in pixels, of the image
*
* Description: Draws an image onto the screen or buffer.
************************************************************************/
{
 int w=width, h=height;

 /* Clip to the left clipping edge */
 if (x < 0) {
	if (x+w < 0)
		return;
	w += x;
	image += -x;
	x = 0;
	}

 /* Clip to the right clipping edge */
 if (x+w >= GFX_SCR_WIDTH) {
	if (x >= GFX_SCR_WIDTH)
		return;
	w = GFX_SCR_WIDTH - x;
	}

 /* Clip to the top clipping edge */
 if (y < 0) {
	if (y+h < 0)
		return;
	h += y;
	image += ((-y) * width);
	y = 0;
	}

 /* Clip to the bottom clipping edge */
 if (y+h >= GFX_SCR_HEIGHT) {
	if (y >= GFX_SCR_HEIGHT)
		return;
	h = GFX_SCR_HEIGHT - y;
	}

 /* Setup address of first pixel to draw */
 buffer += (y * GFX_SCR_WIDTH) + x;

 /* Draw the image scanline by scanline */
 do	{
	memcpy(buffer, image, w);
	buffer += GFX_SCR_WIDTH;
	image += width;
	} while (--h);
}


void GFX_PutImageTrans(unsigned char * buffer, unsigned char * image, int x,
	int y, int width, int height)
/************************************************************************
* Function:    GFX_PutImageTrans
* Parameters:  buffer - pointer to the buffer to draw the image onto
*              image  - pointer to the image data
*              x, y   - coordinates of the upper left-hand corner
*              width  - width, in pixels, of the image
*              height - height, in pixels, of the image
*
* Description: Draws a translucent, alpha blended image onto the screen
*              or buffer.
************************************************************************/
{
 int w=width, h=height,i;

 /* Clip to the left clipping edge */
 if (x < 0) {
	if (x+w < 0)
		return;
	w += x;
	image += -x;
	x = 0;
	}

 /* Clip to the right clipping edge */
 if (x+w >= GFX_SCR_WIDTH) {
	if (x >= GFX_SCR_WIDTH)
		return;
	w = GFX_SCR_WIDTH - x;
	}

 /* Clip to the top clipping edge */
 if (y < 0) {
	if (y+h < 0)
		return;
	h += y;
	image += ((-y) * width);
	y = 0;
	}

 /* Clip to the bottom clipping edge */
 if (y+h >= GFX_SCR_HEIGHT) {
	if (y >= GFX_SCR_HEIGHT)
		return;
	h = GFX_SCR_HEIGHT - y;
	}

 /* Setup address of first pixel to draw */
 buffer += (y * GFX_SCR_WIDTH) + x;

 /* Draw the image scanline by scanline */
 do	{
	for (i=0; i<width; i++) {
		*buffer++ = gfxActiveClut->data[*image++][*buffer];
		}

	buffer += (GFX_SCR_WIDTH - w);
	image += (width - w);
	} while (--h);
}


void GFX_PutImageTransASM(unsigned char * buffer, unsigned char * image,
	int x, int y, int width, int height)
/************************************************************************
* Function:    GFX_PutImageTransASM
* Parameters:  buffer - pointer to the buffer to draw the image onto
*              image  - pointer to the image data
*              x, y   - coordinates of the upper left-hand corner
*              width  - width, in pixels, of the image
*              height - height, in pixels, of the image
*
* Description: Draws a translucent, alpha blended image onto the screen
*              or buffer. This is the asm version.
************************************************************************/
{
 int w=width, h=height;

 /* Clip to the left clipping edge */
 if (x < 0) {
	if (x+w < 0)
		return;
	w += x;
	image += -x;
	x = 0;
	}

 /* Clip to the right clipping edge */
 if (x+w >= GFX_SCR_WIDTH) {
	if (x >= GFX_SCR_WIDTH)
		return;
	w = GFX_SCR_WIDTH - x;
	}

 /* Clip to the top clipping edge */
 if (y < 0) {
	if (y+h < 0)
		return;
	h += y;
	image += ((-y) * width);
	y = 0;
	}

 /* Clip to the bottom clipping edge */
 if (y+h >= GFX_SCR_HEIGHT) {
	if (y >= GFX_SCR_HEIGHT)
		return;
	h = GFX_SCR_HEIGHT - y;
	}

 _asm {
 /* Setup EDI so that it points to the first pixel address. */
	mov  eax,y
	mov  edi,y
	shl  eax,6
	shl  edi,8
	add  edi,eax
	add  edi,buffer
	add  edi,x            /* EDI = ((y<<6) + (y<<8)) + buffer + x */

 /* Store the image pointer in ESI, the width of the scan-line to draw
    in EBX, the active Translucency CLUT in EDX, and zero out EAX */
	mov  esi,image
	mov  ebx,w
	mov  edx,gfxActiveClut
	xor  eax,eax
	cld

 _PutImage_Y_Loop:
	mov  ecx,ebx                 /* ECX = w */

 _PutImage_X_Loop:
	mov  ah,[esi]
	mov  al,[edi]                /* EAX = (image[0]<<8) + buffer[0]; */
	inc  esi
	mov  al,[edx+eax]            /* AL = gfxActiveClut->data[*image][*buffer]; */
	mov  [edi],al                /* Write the alpha blended pixel */
	inc  edi
	dec  ecx
	jnz  _PutImage_X_Loop

	add  edi,320
	add  esi,width
	sub  edi,ebx
	sub  esi,ebx
	dec  h
	jnz  _PutImage_Y_Loop
	};
}


void GFX_PutPixel24(void * buffer24, int x, int y, int color)
/************************************************************************
* Function:    GFX_PutPixel24
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x, y   - coordinates to plot the pixel at
*              color  - color of the pixel to plot
*
* Description: Puts a pixel on the 24-bit screen or off-screen buffer at
*              <X,Y> using the passed color.
************************************************************************/
{
 void * temp_buffer;

 /* Preform clipping on the screen boudaries. */
 if ((x < 0) || (x >= GFX_SCR_WIDTH) || (y < 0) || (y >= GFX_SCR_HEIGHT))
	return;

 /* write the pixel */
 temp_buffer = (void *)((unsigned)buffer24 + (((y * GFX_SCR_WIDTH) + x) * 3));
 ((unsigned short *)temp_buffer)[0] = (unsigned short)color;
 ((unsigned char *)temp_buffer)[2]  = (unsigned char)(color >> 16);
}


void GFX_PutPixelTrans24(void * buffer24, int x, int y, int color, int alpha)
/************************************************************************
* Function:    GFX_PutPixelTrans24
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x, y   - coordinates to plot the pixel at
*              color  - color of the pixel to plot
*              alpha  - blending alpha
*
* Description: Puts a translucent, alpha blended pixel on the 24-bit
*              screen or off-screen buffer at <X,Y> using the passed color.
************************************************************************/
{
 void * temp_buffer;
 int color2;

 /* Preform clipping on the screen boudaries. */
 if ((x < 0) || (x >= GFX_SCR_WIDTH) || (y < 0) || (y >= GFX_SCR_HEIGHT))
	return;

 temp_buffer = (void *)((unsigned)buffer24 + (((y * GFX_SCR_WIDTH) + x) * 3));

 /* First, read the destination pixel that we would normally over-write */
 color2 = ((unsigned long *)temp_buffer)[0];

 /* Blend the two colors together and place the result in color2 */
 color2 = gfxAlphaBlendMap24.blend[alpha](color, color2);

 /* write out the blended pixel */
 ((unsigned short *)temp_buffer)[0] = (unsigned short)color2;
 ((unsigned char *)temp_buffer)[2]  = (unsigned char)(color2 >> 16);
}


void GFX_PutPixelTransASM24(void * buffer24, int x, int y, int color, int alpha)
/************************************************************************
* Function:    GFX_PutPixelTransASM24
* Parameters:  buffer - pointer to the buffer to draw pixel to
*              x, y   - coordinates to plot the pixel at
*              color  - color of the pixel to plot
*              alpha  - blending alpha
*
* Description: Puts a translucent, alpha blended pixel on the 24-bit
*              screen or off-screen buffer at <X,Y> using the passed color.
*              This is the assembly version.
************************************************************************/
{
 /* Preform clipping on the screen boudaries. */
 if ((x < 0) || (x >= GFX_SCR_WIDTH) || (y < 0) || (y >= GFX_SCR_HEIGHT))
	return;


 /* Note that an AGI stall would occur at the "lea edi,[edi+edi*2]", but
    I wanted to keep everything organized for you instead of mixing
    different sections of code around to eliminate the AGI. */

 _asm {
 /* First, setup EDI so that it points to the pixel address */
	mov  edi,y
	mov  eax,y
	shl  edi,6
	shl  eax,8
	add  edi,eax 		    /* EDI = ((y << 6) + (y << 8)); */
	add  edi,x              /* EDI += x; */
	lea  edi,[edi+edi*2]    /* EDI *= 3, AGI stall... :( */
	add  edi,buffer24       /* EDI += buffer; */

 /* Next, setup EBX so that it points to the translucency function map */
	mov  ebx,alpha          /* Setup EBX so that it indexes into function map */
	mov  ebx,dword gfxAlphaBlendMap24[ebx*4]
					    /* EBX = &gfxAlphaBlendMap24.blend[alpha]; */

 /* Finally, preform the actual blending of the pixel colors */
	mov  eax,color          /* Push the color on the stack */
	push eax
	mov  eax,[edi]          /* Read the pixel we would normally overwrite */
	push eax                /* And push it on the stack */
	call ebx                /* Call the blending function */
	add  esp,8              /* Adjust the stack */

 /* And draw out the blended color */
	mov  [edi],ax           /* Write out the lower 2 bytes */
	shr  eax,16             /* Adjust EAX */
	mov  [edi+2],al         /* Write out the top byte */
	};
}



#ifdef __cplusplus
};
#endif

/* end of file */
