/************************************************************************
* File:        PCX.C
* Description: The source to the PCX graphics file loader.
* Author:      Jesse Towner.
************************************************************************/


/* I N C L U D E S *****************************************************/


#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include <stdio.h>
#include <dos.h>

#include "graphics.h"
#include "color.h"
#include "trans.h"
#include "pcx.h"

#ifdef __cplusplus
extern "C" {
#endif


/* S T R U C T U R E S **************************************************/


#pragma pack(1)

/* This is the PCX file's header structure */

typedef struct pcxHeader_t {
	unsigned char  manufacturer;
	unsigned char  version;
	unsigned char  encoding;
	unsigned char  bitsPerPixel;
	unsigned short x, y;
	unsigned short width, height;
	unsigned short horzRes, vertRes;
	unsigned char  egaPalette[48];
	unsigned char  reserved;
	unsigned char  numColorPlanes;
	unsigned short bytesPerScanLine;
	unsigned short paletteType;
	unsigned short horzSize, vertSize;
	unsigned char  padding[54];
	} pcxHeader_t;

#pragma pack(4)


/* F U N C T I O N S ****************************************************/


gfxBitmap_t * PCX_Load(char * filename)
/*************************************************************************
* Function:    PCX_Load
* Parameters:  filename - filename of pcx file to load
* Returns:     Pointer to bitmap if successful, NULL otherwise.
*
* Description: Loads in an 8-bit RLE encoded PCX picture file.
*************************************************************************/
{
 pcxHeader_t pcxHeader;
 unsigned int y, x;
 unsigned char data, size;
 gfxBitmap_t * bitmap;
 FILE * file;

 /* open the pcx file */
 if ((file = fopen(filename, "rb")) == NULL)
	return NULL;

 /* read in the header */
 fread(&pcxHeader, sizeof(pcxHeader_t), 1, file);
 if ((pcxHeader.manufacturer != 10) || (pcxHeader.encoding != 1) ||
	(pcxHeader.bitsPerPixel != 8)) {
	fclose(file);
	return NULL;
	}

 /* Fix up the header a bit... */
 pcxHeader.width = pcxHeader.width - pcxHeader.x + 1;
 pcxHeader.height = pcxHeader.height - pcxHeader.y + 1;
 pcxHeader.bitsPerPixel *= pcxHeader.numColorPlanes;

 /* create the bitmap */
 if ((bitmap = GFX_CreateBitmap(pcxHeader.width, pcxHeader.height, pcxHeader.bitsPerPixel)) == NULL) {
	fclose(file);
	return NULL;
	}

 /* setup the palette with the ega palette values */
 for (y=0; y<16; y++) {
	bitmap->palette[y].r = pcxHeader.egaPalette[y*3] >> 2;
	bitmap->palette[y].g = pcxHeader.egaPalette[y*3+1] >> 2;
	bitmap->palette[y].b = pcxHeader.egaPalette[y*3+2] >> 2;
	}

 /* Load in the RLE encoded pixel data */
 for (y=0; y<bitmap->height; y++) {
	x = 0;

	while (x < bitmap->width) {
		data = (unsigned char)getc(file);
		if ((data & 0xC0) == 0xC0) {
			size = data & 0x3F;
			data = (unsigned char)getc(file);
			}
		else {
			size = 1;
			}

		while (size-- > 0) {
			if (x < bitmap->width)
				((unsigned char *)bitmap->surface)[x + (y * bitmap->width)] = (unsigned char)data;
			x++;
			}
		}
	}

 /* now search for a palette if this is an 8-bit picture */
 if ((pcxHeader.version >= 5) && (pcxHeader.bitsPerPixel == 8)) {
	fseek(file, -769L, SEEK_END);
	if (getc(file) == 12) {
		for (y=0; y<256; y++) {
			bitmap->palette[y].r = (unsigned char)getc(file) >> 2;
			bitmap->palette[y].g = (unsigned char)getc(file) >> 2;
			bitmap->palette[y].b = (unsigned char)getc(file) >> 2;
			}
		}
	}

 /* close the pcx file */
 fclose(file);
 return bitmap;
}


#ifdef __cplusplus
};
#endif

/* end of file */
